/*
 * RayTraceWindow
 *
 */
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import java.io.*;
import java.util.*;

public class RayTraceWindow extends JFrame implements Runnable
{
    // the original joogl scene
    jooglRoot root = null;

    // A joogl clock, can be used to synchronize the joogl and ray
    // tracing windows.   Here, we ray trace the scene at the clock's time.
    // For the animation option, you could also set the time to the current
    // animation frame so the joogl window will follow the animation.
    jooglClock clock = null;
    
    // the scene snapshot, optimized for ray tracing
    jooglRoot rootToTrace = null;

    // the width, height, pixels and Java classes for the image buffer
    int width = 0;
    int height = 0;
    int pixels[];
    MemoryImageSource source;    // used to map an array into an Image
    Image image;		 // the Image, needed for painting and jpeg output
    
    // for animation
    long start;
    long end;
    long increment;
    
    // the panel we render in
    RTImage rt;

    // the ray tracer thread
    Thread traceThread = null;
    
    public class RTImage extends JPanel {
	public RTImage() {
	    setBackground(Color.black);
	}

	public void paintComponent(Graphics g) {
	    super.paintComponent(g);
	    g.drawImage(image, 0,0, this);
	}
    }	

    // common initialization stuff
    public void init(int width, int height, jooglRoot root, jooglClock clock)
    {
        this.width = width;
        this.height = height;
        this.root = root;
        this.clock = clock;

        // set up our pixel array
        this.pixels = new int[width*height];
        // fill with an "unlikely" color
        int c = (new Color(1.0f, 0.333f, 0.0f)).getRGB();
        for (int i = 0; i < width*height; i++) {
            pixels[i] = c;
        }

        // set up pixels for rendering
        source = new MemoryImageSource(width, height, pixels, 0, width);
        source.setAnimated(true);
	image = createImage(source);
        
        // create our previous window
      	rt = new RTImage();
	this.getContentPane().add("Center", rt);
	this.pack();
	this.setSize(width+50,height+50);
	this.setVisible(true);
    }
    
    public RayTraceWindow(int width, int height, jooglRoot root, jooglClock clock)
    {
        super("Rendering Window");
        init(width, height, root, clock);
        
        // Create a thread to perform the raytracing and run it.
        traceThread = new Thread(this);
        traceThread.start();
    }
    
    // use this constructor if you would like to do the animation extra 
    // credit.
    public RayTraceWindow(int width, int height, jooglRoot root, jooglClock clock, 
                          long start, long end, long increment)
    {
        super("Rendering Window");

        this.start = start;
        this.end = end;
        this.increment = increment;

        init(width, height, root, clock);
        
        // Create a thread to perform the raytracing and run it.
        traceThread = new Thread(this);
        traceThread.start();
    }
    
    public Color ShootRay (int x, int y) 
    {
        // Simulation of "work".
        try {Thread.sleep(1);} catch (Exception e) {}
        
	// Just returns a random color.  This is not a very good
	// ray tracer right now.  Your project is to make this function
	// work a lot better.
	return new Color((float)Math.random(), (float)Math.random(), (float)Math.random());
    }

    // the thread mainline to ray trace an image and write the image to disk.
    // to do an animation ray tracer, you would replace this thread run() method
    // with one that uses the MovieEncoder.
    public void run ()
    {
	// Generate a snapshot of the juggler at the time specified by the clock.
        // This is what we'll raytrace.
 	rootToTrace = root.genSnapShot(clock.getTime());
        
        traceImage();
        
        try 
        {
            DataOutputStream out = 
                new DataOutputStream(new BufferedOutputStream(new FileOutputStream("raytrace.jpg")));
            JpegEncoder encoder = new JpegEncoder(image, 100, out);
            encoder.Compress();
            out.close();
        }
        catch(Exception e)
        {
            System.out.println(e);
            System.exit(1);
        }
    }
    
    // The routine to actually do the ray tracing.  Here, you will trace a ray
    // for each image pixel, and store the result in the image buffer
    public void traceImage()
    {
	for (int i = 0; i < width; ++i)
	{	
	    for (int j = 0; j < height; ++j)
            {
		Color c = ShootRay(i,j);
		pixels[i+j*width] = c.getRGB();
	    }
            // only repaint each line, so we don't waste too much effort
            source.newPixels(i,0,1,height);
            rt.repaint();
	}
	System.out.println("Finished ray tracing frame!");
        rt.repaint();
    }
    
    /**
     * Set the root of the scene graph that this class
     * will be rendering.
     */
    public void setRoot(jooglRoot root) {
    	this.root = root;
    }
}
