import java.io.*;

/**
 * The dynamic transform class that is used for frame-based animation.
 * <p>
 * This is a subclass of jooglDynamicTransform.  It stores an array of transforms that represent the different frames.
 * The getValue method returns the transform for the appropriate frame.
 *
 * @version 	1.00
 * @author      Ben Carter
 */
public class jooglFrameBasedTransform extends jooglDynamicTransform {
    private jooglTransform frames[];
    private double millisPerFrame;
    private int numFrames;
    
    public jooglFrameBasedTransform(jooglTransform[] frameTransforms, long framesPerSecond, int frameCount) {
    	millisPerFrame = 1000.0 / framesPerSecond;
    	numFrames = frameCount;
    	frames = frameTransforms;
    }
    
    /** 
     * Get the translation matrix for the passed time.
     *
     * @param time the time in milliseconds at which the property should be evaluated
     * @return a 16 element array holding the current transform
     */
    public double[] getValue(long time) {
	int currFrame = (int)(time / millisPerFrame) % numFrames;
    	setTransform(frames[currFrame]);
    	return matrix;
    }
    
    /**
     * Set the position for each frame
     */
    public void setFrames(jooglTransform[] frameTransforms) {
    	frames = frameTransforms;
    }
    
    /**
     * Set the frame rate in frames per second
     */
    public void setFrameRate(long framesPerSecond) {
    	millisPerFrame = 1000.0 / framesPerSecond;
    }
}
