/**
 * The Visitor subclass that traverses the scene graph of Group nodes looking for 
 * opaque geometry objects to render.
 * <p>
 * Each geometry node encountered is told to render itself if it is not transparent.
 * <p>
 * The necessary properties are pushed when a node is visited and popped when the node is left.
 *
 * @version 	1.00
 * @author      Blair MacIntyre
 */

public class jooglRenderVisitor extends jooglVisitor {
    /** The default constructor */
    public jooglRenderVisitor ()
    {
        super();
    }

    /** 
     * The action to be done when a node is first visited. 
     *
     * @param node the current node being visited
     */
    public void preAction(jooglNode node)
    {
    	// push the properties we care about for rendering
        if (node.transform != null)
            node.transform.push();
        if (node.material != null)
            node.material.push();
        if (node.drawMode != null)
            node.drawMode.push();
        if (node.shadeMode != null)
            node.shadeMode.push();
            
        // if a geometry node, render it
        if (node instanceof jooglGeometry) 
        {
            jooglMaterial mat = (jooglMaterial)(jooglProperty.peek(jooglProperty.PROP_MATERIAL));
            if ((mat != null) && (!mat.isTransparent()))
                ((jooglGeometry)node).render();
        }
    }
        
    /** 
     * The action to be done when a node has been visited and is being left. 
     *
     * @param node the current node being visited
     */
    public void postAction(jooglNode node)
    {
        if (node.transform != null)
            node.transform.pop();
        if (node.material != null)
            node.material.pop();
        if (node.drawMode != null)
            node.drawMode.pop();
        if (node.shadeMode != null)
            node.shadeMode.pop();
    }
}
